# Generated by Django 2.1.7 on 2019-03-02 17:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_verifier', '0004_auto_20190225_2153'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dataverificationtag',
            name='verification_state',
            field=models.CharField(choices=[('pending', 'pending'), ('verified', 'verified'), ('invalid', 'invalid'), ('rejected', 'rejected')], default='pending', max_length=10),
        ),
    ]
