# Generated by Django 2.2 on 2019-04-04 21:16

import django.core.validators
from django.db import migrations, models

import spkcspider.apps.spider.validators


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0011_auto_20190331_1222'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='assignedcontent',
            name='fake_id',
        ),
        migrations.AddField(
            model_name='assignedcontent',
            name='description',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AddField(
            model_name='assignedcontent',
            name='name',
            field=models.CharField(blank=True, default='', max_length=255, validators=[
                spkcspider.apps.spider.validators.content_name_validator
            ]),
        ),
        migrations.AlterField(
            model_name='usercomponent',
            name='name',
            field=models.CharField(db_index=True, help_text='Name of the component.<br/>Note: there are special named components with different protection types and scopes.<br/>Most prominent: "index" for authentication', max_length=255, validators=[django.core.validators.RegexValidator('^(\\w[\\w ]*\\w|\\w?)$')]),
        ),
    ]
