# Generated by Django 2.2 on 2019-04-14 17:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0015_auto_20190414_1743'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assignedcontent',
            name='attached_to_token',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='spider_base.AuthToken'),
        ),
    ]
