# Generated by Django 2.1.3 on 2018-11-21 07:38

from django.db import migrations


class Migration(migrations.Migration):
    """
        Introduced new field which was not required after integrating the hashes
        in the graph file
    """
    dependencies = [
        ('spider_verifier', '0002_auto_20181031_2121'),
    ]

    operations = [
    ]
