# Generated by Django 2.2.1 on 2019-05-12 16:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0021_auto_20190511_1046'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='authtoken',
            name='created_by_special_user',
        ),
        migrations.AlterField(
            model_name='travelprotection',
            name='login_protection',
            field=models.CharField(choices=[('a', 'Hide'), ('b', 'Hide if triggered'), ('c', 'Disable login'), ('d', 'Wipe'), ('e', 'Wipe User')], default='a', max_length=1),
        ),
    ]
