# Generated by Django 2.1.7 on 2019-03-09 22:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_verifier', '0005_auto_20190302_1741'),
    ]

    operations = [
        migrations.AddField(
            model_name='verifysourceobject',
            name='update_secret',
            field=models.CharField(blank=True, max_length=120, null=True, unique=True),
        ),
    ]
