# Generated by Django 2.2 on 2019-04-14 14:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_tags', '0006_auto_20190226_1853'),
    ]

    operations = [
        migrations.AddField(
            model_name='taglayout',
            name='description',
            field=models.TextField(blank=True, default=''),
        ),
    ]
