# Generated by Django 2.1.2 on 2018-11-11 13:16

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spider_verifier', '0003_dataverificationtag_linked_hashes'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='dataverificationtag',
            name='linked_hashes',
        ),
    ]
