# Generated by Django 2.2 on 2019-04-07 14:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('spider_base', '0013_auto_20190407_1313'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='assignedcontent',
            unique_together={('content_type', 'object_id')},
        ),
        migrations.AddConstraint(
            model_name='assignedcontent',
            constraint=models.UniqueConstraint(fields=('usercomponent', 'info'), name='unique_info'),
        ),
    ]
