# Generated by Django 2.2.1 on 2019-05-30 16:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_verifier', '0006_verifysourceobject_update_secret'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='verifysourceobject',
            name='update_secret',
        ),
        migrations.AddField(
            model_name='verifysourceobject',
            name='token',
            field=models.CharField(blank=True, max_length=126, null=True, unique=True),
        ),
    ]
