# Generated by Django 2.1.7 on 2019-02-21 22:53

from django.db import migrations, models
import django.db.models.deletion
import spkcspider.apps.spider_keys.models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_keys', '0003_auto_20190130_1137'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='publickey',
            name='use_for_encryption',
        ),
        migrations.AlterField(
            model_name='anchorkey',
            name='key',
            field=models.OneToOneField(help_text='"Public Key"-Content for signing identifier. It is recommended to use different keys for signing and encryption.', on_delete=django.db.models.deletion.CASCADE, related_name='anchorkey', to='spider_keys.PublicKey'),
        ),
        migrations.AlterField(
            model_name='publickey',
            name='key',
            field=models.TextField(help_text='It is recommended to use different keysfor signing and encryption', validators=[spkcspider.apps.spider_keys.models.valid_pkey_properties]),
        ),
    ]
