# Generated by Django 2.1.5 on 2019-01-30 11:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spider_keys', '0002_anchorkey_anchorserver'),
    ]

    operations = [
        migrations.AddField(
            model_name='publickey',
            name='use_for_encryption',
            field=models.BooleanField(blank=True, default=True, help_text='Use for encryption not for signing.'),
        ),
        migrations.AlterField(
            model_name='anchorkey',
            name='key',
            field=models.OneToOneField(help_text='"Public Key"-Content', limit_choices_to={'use_for_encryption': False}, on_delete=django.db.models.deletion.CASCADE, related_name='anchorkey', to='spider_keys.PublicKey'),
        ),
    ]
