# Generated by Django 2.2 on 2019-04-05 00:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spider_filets', '0010_auto_20190403_0417'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='filefilet',
            name='name',
        ),
        migrations.RemoveField(
            model_name='textfilet',
            name='name',
        ),
        migrations.RemoveField(
            model_name='textfilet',
            name='preview_words',
        ),
    ]
